==> dasu_4.11.jar: edu/northwestern/dasu/dynamic/measurement/DasuBrowserModule.java <==
/**
 * Dasu Project
 *
 * File:         DasuNdtModule.java
 * RCS:          $Id$
 * Description:  OptimalEdge class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 15, 2008 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.dynamic.measurement;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.*;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.dynamic.drools.FactBrowserProbeResult;
import edu.northwestern.dasu.measurement.DasuProbeModule;
import edu.northwestern.dasu.measurement.tasks.ProbeTask;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.netinterface.NetInterfaceClassifier;
import edu.northwestern.dasu.netinterface.NetInterfaceClassifier.InterfaceType;
import edu.northwestern.dasu.util.Pair;
import edu.northwestern.dasu.util.Util;
import edu.northwestern.dasu.util.json.JSONValue;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The DasuBrowserModule class manages the set of NDT probes performed
 *         by the peer.
 */

public class DasuBrowserModule extends DasuProbeModule<FactBrowserProbeResult> {

	private final static long waitTimeBetweenRepeats = 5 * 1000; // 
	public Hashtable<Long, FactBrowserProbeResult> browserResultTable;
	private ArrayList<ProbeSubTask> toDoUrlList = new ArrayList<ProbeSubTask>();
	private static DasuBrowserModule self;
	private static LinkedList<String> servers = new LinkedList<String>();	
	public Map<String, Object> params = null;

	public DasuBrowserModule() {

		super("DasuBrowserModule", 2.0f);
		this.pi = Main.getPluginInterface();
		browserResultTable = new Hashtable<Long, FactBrowserProbeResult>();
		active = true;

	}

	public synchronized static DasuBrowserModule getInstance() {
		if (self == null) {
			self = new DasuBrowserModule();
			self.setDaemon(true);
		}
		return self;
	}

	@Override
	public void run() {
		while (!Main.isShuttingDown() && isActive()) {
			ProbeSubTask pst = null;
			synchronized (toDoUrlList) {
				while ((toDoUrlList.size() == 0) && !Main.isShuttingDown()) {
					try {
						toDoUrlList.wait(sleepInterval * 1000);
					} catch (InterruptedException e) {
						setActive(false);
						LOGGER.info("STOP: Adios...");
						return;
					}
				}

				if (toDoUrlList.size() > 0){
					pst = toDoUrlList.remove(0);
				}
			}
			if (pst != null) {
				while (isPause() && isActive()) {
					try {
						Thread.sleep(pauseInterval * 1000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}

				Object ret = probeIp(pst);
				if (ret != null) {

					FactBrowserProbeResult result = (FactBrowserProbeResult) ret;
					result.setPst(pst);
					HashMap<String, Object> map = dumpObj(result);
					String res = JSONValue.toJSONString(map);
					LOGGER.info("BROWSER_MODULE: Reporting string: "+res);
					//AMCoordinatorV2.commitFeedback("result-browser", null, null, "", res);
					//AMCoordinatorV2.commitFeedback(type, name, email, message, dump);
					updateBrowserResultQueue(result);

					try {
						Thread.sleep(waitTimeBetweenRepeats);
					}catch (Exception e) {
						//
					}

					if (yieldInterval != -1) {
						try {
							sleep(yieldInterval);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			}

		} // end while active

		LOGGER.info("STOP: Adios...");
	}

	@Override
	public void probeAll(ProbeTask pt) {
		if (pt != null) {
			synchronized (toDoUrlList) {
				for (ProbeSubTask pst : pt.getSubTasks()) {
					if (pst.ipAddress == null) {
						pst.ipAddress = "";
					}
					toDoUrlList.add(pst);
				}
				toDoUrlList.notifyAll();
			}
		}
	}

	@Override
	public void probeWithPriority(ProbeSubTask pst) {
		if (pst.ipAddress == null) {
			pst.ipAddress = "";
		}

		synchronized (toDoUrlList) {
			toDoUrlList.add(pst);
			toDoUrlList.notifyAll();
		}
	}

	@Override
	public void probe(ProbeSubTask pst) {
		if (pst != null) {
			if (pst.ipAddress == null)
				pst.ipAddress = "";

			synchronized (toDoUrlList) {
				toDoUrlList.add(pst);
				toDoUrlList.notifyAll();
			}
		}
	}

	@Override
	protected Object probeIp(ProbeSubTask pst) {
		
		String interfaceName = Main.getInterfaceName();
		NetInterfaceClassifier interfaceClassifier = new NetInterfaceClassifier(interfaceName);
		InterfaceType interfaceType = interfaceClassifier.getInterfaceType();
		if (!isActive())
			return null;

		if (pst.ipAddress.equals("")) {
			return null;
		}

		long timestamp = Util.currentGMTTime();
		boolean error = false;

		// Calculate page rendering time (first try, no Javascript)
		Long time1 = browse(pst.ipAddress, false);
		LOGGER.info("BROWSER_MODULE: " + pst.ipAddress + " - Page loading time 1: " + time1);
		// Check for error
		if(time1 < 0){ error = true;}

		// Take a short break
		try {
			Thread.sleep(waitTimeBetweenRepeats);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		// Calculate page rendering time (second try, no Javascript)
		Long time2 = browse(pst.ipAddress, false);
		LOGGER.info("BROWSER_MODULE: " + pst.ipAddress + " - Page loading time 2: " + time2);
		// Check for error
		if(time2 < 0){ error = true;}

		// Take a short break
		try {
			Thread.sleep(waitTimeBetweenRepeats);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}


		// Calculate page loading time (with Javascript)
		Long time3 = browse(pst.ipAddress, true);
		LOGGER.info("BROWSER_MODULE: " + pst.ipAddress + " - Page loading time 3: " + time3);
		// Check for error
		if(time3 < 0){ error = true;}

		// Record results
		FactBrowserProbeResult ret = new FactBrowserProbeResult(pst.ipAddress, -1,
				timestamp, interfaceName, interfaceType.toString(), browserType, 0, time1, time2, time3, error);
		ret.setPst(pst);
		return ret;

	}

	Shell shell = null;
	private long start = -1;
	private long end = -1;
	private boolean finished = false;
	private String browserType = "";

	private long browse(final String url, final boolean javascriptEnabled) {
		LOGGER.info("BROWSER_MODULE: Loading page "+url+" ...");
		finished = false;
		end = -1;
		start = -1;
				
		Display.getDefault().asyncExec(new Runnable() {
			
			public void run() {
				if (shell != null) {
					try {
						shell.dispose();
					} catch (Exception e){

					}
				}
				
				shell = new Shell(Display.getDefault());
				shell.setText("Dasu Browser");
				shell.setSize(800, 800);
				shell.setVisible(false);
				shell.setLayout(new FillLayout());
				
				Browser browser = new Browser(shell, SWT.NONE);		
				browser.setJavascriptEnabled(javascriptEnabled);
				browserType = browser.getBrowserType();
				browser.setVisible(false);
				browser.setEnabled(false);
				
				browser.addProgressListener(new ProgressListener() {
					public void changed(ProgressEvent event) {
					}

					public void completed(ProgressEvent event) {
						end = System.currentTimeMillis();
						finished = true;
						try{
							shell.dispose();
						} catch (Exception e){
						}
					}

				});

				finished = false;
				start = System.currentTimeMillis();
				browser.setUrl(url);

			}
		});
		
		while (!finished) {
			if(shell != null && shell.isDisposed()){
				if(finished == false){
					finished = true;
					end = -1;
				}
			}
			try {
				Thread.sleep(1 * 1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}

		if(end < 0 || start < 0){
			return -1;
		}
		return end - start;
	}

	private void updateBrowserResultQueue(FactBrowserProbeResult result) {

		Long id = generateId();
		browserResultTable.put(id, result);
		updateResultQueue(result, result.getProbeType(), id);

	}

	public static LinkedList<String> getServers() {
		return servers;
	}

	@Override
	public void stopModule() {

		LOGGER.info("STOP: stopping DasuNdtModule...");
		setActive(false);
		synchronized (toDoUrlList) {
			toDoUrlList.notify();
		}
	}

	public Pair<String, FactTemplate> proccessResult(long eventId) {
		FactBrowserProbeResult browserResult = browserResultTable.remove(eventId);
		return new Pair<String, FactTemplate>(browserResult.url, browserResult);

	}



	public HashMap<String, Object> dumpObj(Object obj) {
		HashMap<String, Object> ret = new HashMap<String, Object>();

		Method[] methods = obj.getClass().getMethods();
		for(int i = 0; i < methods.length; i++) {
			Method m = methods[i];
			String name = m.getName();

			String field = null;

			if(name.startsWith("get")) {
				field = name.substring(3, 4).toLowerCase() + name.substring(4);
			} else if(name.startsWith("is")) {
				field = name.substring(2, 3).toLowerCase() + name.substring(3);
			}

			if(field == null) continue;
			if(field.equals("class")) continue;
			if(field.equals("field")) continue;

			try {
				ret.put(field, m.invoke(obj, new Object[0]));
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
			}
		}

		return ret;
	}


}

==> dasu_5.31.jar: edu/northwestern/dasu/dynamic/measurement/DasuBrowserModule.java <==
/**
 * Dasu Project
 *
 * File:         DasuNdtModule.java
 * RCS:          $Id$
 * Description:  OptimalEdge class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 15, 2008 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.dynamic.measurement;

import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.*;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.dynamic.drools.FactBrowserProbeResult;
import edu.northwestern.dasu.measurement.DasuProbeModule;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.netinterface.NetInterfaceClassifier;
import edu.northwestern.dasu.netinterface.NetInterfaceClassifier.InterfaceType;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The DasuBrowserModule class manages the set of NDT probes performed
 *         by the peer.
 */

public class DasuBrowserModule extends DasuProbeModule<FactBrowserProbeResult> {

	private final static long waitTimeBetweenRepeats = 5 * 1000; // 
	private static DasuBrowserModule self;	
	public Map<String, Object> params = null;

	public DasuBrowserModule() {
		super("DasuBrowserModule", 2.0f);
	}

	public synchronized static DasuBrowserModule getInstance() {
		if (self == null) {
			self = new DasuBrowserModule();
			self.setDaemon(true);
		}
		return self;
	}
	
	public boolean isIdle() {
		return idle;
	}

	@Override
	protected FactBrowserProbeResult probeIp(ProbeSubTask pst) {
		
		String interfaceName = Main.getInterfaceName();
		NetInterfaceClassifier interfaceClassifier = new NetInterfaceClassifier(interfaceName);
		InterfaceType interfaceType = interfaceClassifier.getInterfaceType();
		if (!isActive())
			return null;

		if (pst.getIpAddress().equals("")) {
			return null;
		}

		long timestamp = Util.currentGMTTime();
		boolean error = false;

		// Calculate page rendering time (first try, no Javascript)
		Long time1 = browse(pst.getIpAddress(), false);
		LOGGER.info("BROWSER_MODULE: " + pst.getIpAddress() + " - Page loading time 1: " + time1);
		// Check for error
		if(time1 < 0){ error = true;}

		// Take a short break
		try {
			Thread.sleep(waitTimeBetweenRepeats);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		// Calculate page rendering time (second try, no Javascript)
		Long time2 = browse(pst.getIpAddress(), false);
		LOGGER.info("BROWSER_MODULE: " + pst.getIpAddress() + " - Page loading time 2: " + time2);
		// Check for error
		if(time2 < 0){ error = true;}

		// Take a short break
		try {
			Thread.sleep(waitTimeBetweenRepeats);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}


		// Calculate page loading time (with Javascript)
		Long time3 = browse(pst.getIpAddress(), true);
		LOGGER.info("BROWSER_MODULE: " + pst.getIpAddress() + " - Page loading time 3: " + time3);
		// Check for error
		if(time3 < 0){ error = true;}

		// Record results
		FactBrowserProbeResult ret = new FactBrowserProbeResult(pst.getIpAddress(), -1,
				timestamp, interfaceName, interfaceType.toString(), browserType, 0, time1, time2, time3, error);
		ret.setPst(pst);
		return ret;

	}

	Shell shell = null;
	private long start = -1;
	private long end = -1;
	private boolean finished = false;
	private String browserType = "";

	private long browse(final String url, final boolean javascriptEnabled) {
		LOGGER.info("BROWSER_MODULE: Loading page "+url+" ...");
		finished = false;
		end = -1;
		start = -1;
				
		Display.getDefault().asyncExec(new Runnable() {
			
			public void run() {
				if (shell != null) {
					try {
						shell.dispose();
					} catch (Exception e){

					}
				}
				
				shell = new Shell(Display.getDefault());
				shell.setText("Dasu Browser");
				shell.setSize(800, 800);
				shell.setVisible(false);
				shell.setLayout(new FillLayout());
				
				Browser browser = new Browser(shell, SWT.NONE);		
				browser.setJavascriptEnabled(javascriptEnabled);
				browserType = browser.getBrowserType();
				browser.setVisible(false);
				browser.setEnabled(false);				
				
				browser.addProgressListener(new ProgressListener() {
					public void changed(ProgressEvent event) {
					}

					public void completed(ProgressEvent event) {
						end = System.currentTimeMillis();
						finished = true;
						try{
							shell.dispose();
						} catch (Exception e){
						}
					}

				});

				finished = false;
				start = System.currentTimeMillis();
				browser.setUrl(url);

			}
		});
		
		while (!finished) {
			if(shell != null && shell.isDisposed()){
				if(finished == false){
					finished = true;
					end = -1;
				}
			}
			try {
				Thread.sleep(1 * 1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}

		if(end < 0 || start < 0){
			return -1;
		}
		return end - start;
	}

	@Override
	public Integer getNumberActiveProbes() {
		
		if (!isIdle())
			return 1;
		
		return 0;
		
	}

}
